// src/eq.js
var eq = (
  /** @return {boolean} */
  (x, y) => {
    if (x === y) {
      return true;
    }
    const xConstructor = x?.constructor;
    if (xConstructor === y?.constructor) {
      if (xConstructor === Number) {
        return isNaN(x) && isNaN(y) || +x === +y;
      }
      if (xConstructor === Date) {
        return +x === +y;
      }
      if (xConstructor === String || xConstructor === RegExp) {
        return "" + x === "" + y;
      }
      if (xConstructor === Array) {
        return x.length === y.length && x.every((item, index) => eq(item, y[index]));
      }
      if (xConstructor === Object) {
        const keysOfX = Object.keys(x);
        return keysOfX.length === Object.keys(y).length && keysOfX.every((key) => eq(x[key], y[key]));
      }
      if (xConstructor === Set || xConstructor === Map) {
        if (x.size !== y.size) {
          return false;
        }
        const xa = [...x];
        const ya = [...y];
        return xa.every((xv) => ya.find((yv) => eq(xv, yv)));
      }
    }
    return false;
  }
);

// src/is.js
var is = (type, arg) => arg?.constructor === type;

// src/h.js
var _NS = {
  xlink: "http://www.w3.org/1999/xlink"
};
var _h = (namespaceURI) => {
  const createElement = namespaceURI == null ? (tag) => document.createElement(tag) : (tag) => document.createElementNS(namespaceURI, tag);
  const h2 = (tagOrNode, ...args) => {
    const node = is(String, tagOrNode) ? createElement(tagOrNode) : tagOrNode;
    for (const arg of args) {
      let child = null;
      if (arg instanceof Node) {
        child = arg;
      } else if (is(String, arg) || is(Number, arg)) {
        child = new Text(arg);
      } else if (is(Array, arg)) {
        child = h2(...arg);
      } else if (arg != null) {
        for (const name in arg) {
          const value = arg[name];
          if (name[0] === "$") {
            const name1 = name.slice(1);
            if (is(Object, value)) {
              node[name1] = node[name1] ?? {};
              Object.assign(node[name1], value);
            } else {
              node[name1] = value;
            }
          } else if (node instanceof Element) {
            const indexOfColon = name.indexOf(":");
            if (indexOfColon >= 0) {
              const ns = _NS[name.slice(0, indexOfColon)];
              if (ns != null) {
                const basename = name.slice(indexOfColon + 1);
                if (value === true) {
                  node.setAttributeNS(ns, basename, "");
                } else if (value === false) {
                  node.removeAttributeNS(ns, basename);
                } else {
                  node.setAttributeNS(ns, basename, is(String, value) ? value : "" + value);
                }
              }
            } else {
              if (value === true) {
                node.setAttribute(name, "");
              } else if (value === false) {
                node.removeAttribute(name);
              } else {
                node.setAttribute(name, is(String, value) ? value : "" + value);
              }
            }
          }
        }
      }
      if (child != null) {
        node.appendChild(child);
      }
    }
    return node;
  };
  return h2;
};
var h = _h();
var s = _h("http://www.w3.org/2000/svg");

// src/chartable.js
var _COLORS = ["#e22", "#e73", "#fc3", "#ad4", "#4d9", "#3be", "#45d", "#c3e"];
var chartable = ({
  bottom = 50,
  gapX = 70,
  gapY = 30,
  headerColumn = false,
  headerRow = true,
  id,
  left = 200,
  maxY = 10,
  right = 200,
  singleScale = false,
  table,
  title,
  top = 70,
  xLabels = [],
  zLabels = [],
  zxY = []
} = {}) => {
  if (table != null) {
    const zxYNotPassed = !zxY.length;
    const xLabelsNotPassed = !xLabels.length;
    const zLabelsNotPassed = !zLabels.length;
    table.querySelectorAll("tr").forEach((row, r) => row.querySelectorAll("td,th").forEach((col, c) => {
      const x = r - +headerRow;
      const z = c - +headerColumn;
      const value = col.innerText;
      if (x >= 0 && z >= 0 && zxYNotPassed) {
        zxY[z] = zxY[z] ?? [];
        zxY[z][x] = parseFloat(value);
      } else if (x >= 0 && z < 0 && xLabelsNotPassed) {
        xLabels[x] = value;
      } else if (x < 0 && z >= 0 && zLabelsNotPassed) {
        zLabels[z] = value;
      }
    }));
  }
  let bestScales = [Infinity, -Infinity, Infinity, -Infinity, Infinity];
  const ranges = zxY.map((xY) => {
    xY = xY.filter((y) => !isNaN(y));
    return [Math.min(...xY), Math.max(...xY)];
  });
  if (singleScale) {
    bestScales = ranges.reduce(([totalMin, totalMax], [min, max]) => [Math.min(totalMin, min), Math.max(totalMax, max)], bestScales);
  } else {
    ranges.forEach(([min12, max0]) => ranges.forEach(([min0, max12]) => {
      if (min0 >= min12 && max12 >= max0) {
        let min22 = Infinity;
        let max22 = -Infinity;
        ranges.forEach(([min, max]) => {
          if (min < min12 || max12 < max) {
            if (min22 > min) {
              min22 = min;
            }
            if (max22 < max) {
              max22 = max;
            }
          }
        });
        let cost = 0;
        ranges.forEach(([min, max]) => {
          if (min < max) {
            cost += 1 - (max - min) / (min12 <= min && max <= max12 ? max12 - min12 : max22 - min22);
          }
        });
        if (bestScales[4] > cost) {
          bestScales = [min12, max12, min22, max22, cost];
        }
      }
    }));
  }
  const [min1, max1, min2, max2] = bestScales;
  const maxX = zxY[0].length;
  const xi = Array.from({ length: maxX }, (_, x) => x * gapX);
  const yi = Array.from({ length: maxY }, (_, y) => y * gapY);
  const w = gapX * (maxX - 1);
  const h2 = gapY * (maxY - 1);
  const svgW = left + w + right;
  const svgH = top + h2 + bottom;
  const graph = [
    "g",
    { transform: `translate(${left} ${top})` },
    ...yi.map((y) => ["line", { x1: 0, x2: w, y1: y, y2: y, stroke: "#8888" }]),
    ...xi.map((x) => ["line", { x1: x, x2: x, y1: 0, y2: h2, stroke: "#8888" }])
  ];
  const lColors = ["g", { class: "chartable-z-colors", transform: "translate(-80 0)" }];
  const lLabels = ["g", { class: "chartable-z-labels", transform: "translate(-95 0)" }];
  const rColors = ["g", { class: "chartable-z-colors", transform: "translate(80 0)" }];
  const rLabels = ["g", { class: "chartable-z-labels", transform: "translate(95 0)" }];
  const yLabel = (min, max, y) => (min + (max - min) * (maxY - 1 - y) / (maxY - 1)).toFixed(2).replace(/\.?0+$/, "");
  zxY.forEach((xa, z) => {
    const cls = `chartable-z-${z + 1}`;
    const fill = _COLORS[z % _COLORS.length];
    let [min, max] = ranges[z];
    if (min1 <= min && max <= max1) {
      min = min1;
      max = max1;
      lColors.push(["circle", { class: cls, cx: 0, cy: gapY * (lColors.length - 2), r: 5, fill }]);
      lLabels.push([
        "text",
        { x: 0, y: gapY * (lLabels.length - 2), "text-anchor": "end", "alignment-baseline": "middle" },
        zLabels[z]
      ]);
    } else {
      min = min2;
      max = max2;
      rColors.push(["circle", { class: cls, cx: 0, cy: gapY * (rColors.length - 2), r: 5, fill }]);
      rLabels.push([
        "text",
        { x: 0, y: gapY * (rLabels.length - 2), "text-anchor": "start", "alignment-baseline": "middle" },
        zLabels[z]
      ]);
    }
    const y = xa.map((x) => isNaN(x) ? x : h2 * (max - x) / (max - min));
    xi.forEach((x, i) => {
      if (!isNaN(y[i])) {
        graph.push(["circle", { class: cls, cx: x, cy: y[i], r: 5, fill }]);
        if (!isNaN(y[i - 1])) {
          graph.push(["line", { class: cls, x1: x, x2: xi[i - 1], y1: y[i], y2: y[i - 1], stroke: fill }]);
        }
      }
    });
  });
  return s(
    "svg",
    {
      viewBox: `0 0 ${svgW} ${svgH}`,
      width: `${svgW}px`,
      height: `${svgH}px`,
      class: "chartable"
    },
    id != null ? { id } : null,
    title != null ? [
      "text",
      { class: "chartable-title", x: svgW / 2, y: top / 2, "text-anchor": "middle", "alignment-baseline": "middle" },
      title
    ] : null,
    graph,
    min1 < Infinity ? [
      "g",
      { class: "chartable-left", transform: `translate(${left} ${top})` },
      [
        "g",
        { class: "chartable-y-labels", transform: "translate(-15 0)" },
        ...yi.map((y, i) => [
          "text",
          { x: 0, y, "text-anchor": "end", "alignment-baseline": "middle" },
          yLabel(min1, max1, i)
        ])
      ],
      lColors,
      lLabels
    ] : null,
    min2 < Infinity ? [
      "g",
      { class: "chartable-right", transform: `translate(${left + w} ${top})` },
      [
        "g",
        { class: "chartable-y-labels", transform: "translate(15 0)" },
        ...yi.map((y, i) => [
          "text",
          { x: 0, y, "text-anchor": "start", "alignment-baseline": "middle" },
          yLabel(min2, max2, i)
        ])
      ],
      rColors,
      rLabels
    ] : null,
    xLabels.length > 0 ? [
      "g",
      { transform: `translate(${left} ${top + h2})` },
      [
        "g",
        { class: "chartable-x-labels", transform: "translate(0 15)" },
        ...xi.map((x, i) => [
          "text",
          { x, y: 0, "text-anchor": "middle", "alignment-baseline": "hanging" },
          xLabels[i]
        ])
      ]
    ] : null
  );
};

// src/escape.js
var escapeValues = (escapeMap, values) => values.map((value) => (escapeMap.get(value?.constructor) ?? escapeMap.get(void 0))?.(value) ?? "");
var escape = (escapeMap, template, ...values) => String.raw(template, ...escapeValues(escapeMap, values));

// src/has.js
var has = (key, ref) => (is(String, key) || is(Number, key) || is(Symbol, key)) && Object.hasOwnProperty.call(ref ?? Object, key);

// src/fixTypography.js
var _TAGS_TO_SKIP = { IFRAME: 1, NOSCRIPT: 1, PRE: 1, SCRIPT: 1, STYLE: 1, TEXTAREA: 1 };
var fixTypography = (node) => {
  const queue = [node];
  while (queue.length > 0) {
    const node0 = queue.shift();
    if (node0 instanceof Element) {
      for (let i = 0; i < node0.childNodes.length; ++i) {
        const childNode = node0.childNodes[i];
        if (childNode instanceof Text) {
          queue.push(childNode);
        } else if (childNode instanceof Element && !has(childNode.tagName, _TAGS_TO_SKIP)) {
          queue.push(childNode);
        }
      }
    } else if (node0 instanceof Text && node0.nodeValue != null && node0.nodeValue.trim() !== "") {
      let previousNode = node0;
      node0.nodeValue.split(/(\s|\(|„)([aiouwz—]\s)/gi).forEach((chunk, i) => {
        i %= 3;
        const currentNode = i === 2 ? h("span", { style: "white-space:nowrap" }, chunk) : i === 1 ? document.createTextNode(chunk) : document.createTextNode(chunk.replace(/(\/(?=[^/\s])|\.(?=[^\s]))/g, "$1\u200B"));
        if (node0.parentNode != null) {
          node0.parentNode.insertBefore(currentNode, previousNode.nextSibling);
        }
        previousNode = currentNode;
      });
      node0.parentNode?.removeChild(node0);
    }
  }
};

// src/jcss.js
var _jcss = (node, prefix, result, split) => {
  const queue = [[node, prefix]];
  while (queue.length) {
    const [style2, prefix2] = queue.shift() ?? [];
    if (style2 == null || prefix2 == null) {
      continue;
    }
    if (is(Array, style2)) {
      result.push(prefix2, prefix2 !== "" ? "{" : "", style2.join(";"), prefix2 !== "" ? "}" : "");
    } else {
      const todo = [];
      let attributes = [];
      let attributesPushed = false;
      for (const key in style2) {
        const value = style2[key];
        if (is(String, value) || is(Number, value)) {
          if (!attributesPushed) {
            attributesPushed = true;
            attributes = [];
            todo.push([attributes, prefix2]);
          }
          attributes.push(`${split(key).replace(/([A-Z])/g, (_, letter) => "-" + letter.toLowerCase())}:${value}`);
        } else {
          attributesPushed = false;
          const newPrefix = [];
          const keySplitted = key.split(",");
          for (const prefixItem of prefix2.split(",")) {
            for (const keyItem of keySplitted) {
              newPrefix.push(prefixItem + keyItem);
            }
          }
          todo.push([value, newPrefix.join(",")]);
        }
      }
      queue.unshift(...todo);
    }
  }
};
var jcss = (root, splitter = "$$") => {
  const split = (text) => text.split(splitter)[0];
  const result = [];
  for (const key in root) {
    const value = root[key];
    if (key[0] === "@") {
      result.push(split(key) + "{");
      _jcss(value, "", result, split);
      result.push("}");
    } else {
      _jcss(value, split(key), result, split);
    }
  }
  return result.join("");
};

// src/jsOnParse.js
var jsOnParse = (handlers, text) => JSON.parse(text, (key, value) => {
  if (is(Object, value)) {
    let isSecondKey = false;
    for (key in value) {
      if (isSecondKey) {
        return value;
      }
      isSecondKey = true;
    }
    if (has(key, handlers) && is(Array, value[key])) {
      return handlers[key](...value[key]);
    }
  }
  return value;
});

// src/locale.js
var _locale = (locales, language, text, version) => {
  const v = locales?.[language]?.[version]?.[text];
  if (is(String, v)) {
    return v;
  }
  const t = locales?.[language]?.[text];
  return is(String, t) ? t : text;
};
var locale = (locales, defaultLanguage, languages = navigator.languages) => {
  for (const language of languages) {
    if (has(language, locales)) {
      return _locale.bind(0, locales, language);
    }
  }
  return _locale.bind(0, locales, defaultLanguage);
};

// src/nanolight.js
var nanolight = (pattern, highlighters, code) => {
  const result = [];
  code.split(pattern).forEach((chunk, index) => {
    index %= highlighters.length;
    if (chunk != null) {
      result.push(highlighters[index](chunk, index));
    }
  });
  return result;
};

// src/nanolightJs.js
var nanolightJs = nanolight.bind(
  0,
  /('.*?'|".*?"|`[\s\S]*?`)|(\/\/.*?\n|\/\*[\s\S]*?\*\/)|(break|case|catch|const|continue|debugger|default|delete|do|else|eval|export\s+type|export|extends|false|finally|for|from|function|goto|if|import|in|instanceof|is|keyof|let|NaN|new|null|package|return|super|switch|this|throw|true|try|typeof|undefined|var|void|while|with|yield)(?!\w)|([<>=.?:&|!~*/%+-])|(0x[\dabcdef]+|0o[01234567]+|0b[01]+|\d+(?:\.[\d_]+)?(?:e[+-]?[\d_]+)?)|([$\w]+)(?=\()|([$\wąćęłńóśżźĄĆĘŁŃÓŚŻŹ]+)/,
  [
    (chunk) => chunk,
    // no match
    (chunk) => ["u", chunk],
    // string literals
    (chunk) => ["em", chunk],
    // comments
    (chunk) => ["b", chunk],
    // keywords
    (chunk) => ["b", chunk],
    // operators
    (chunk) => ["u", chunk],
    // number literals
    (chunk) => ["ins", chunk],
    // function calls
    (chunk) => ["i", chunk]
    // literals
  ]
);

// src/plUral.js
var plUral = (singular, plural2, plural5, value) => {
  const absValue = Math.abs(value);
  const absValueMod10 = absValue % 10;
  return value === 1 ? singular : absValue === 12 || absValue === 13 || absValue === 14 ? plural5 : absValueMod10 === 2 || absValueMod10 === 3 || absValueMod10 === 4 ? plural2 : plural5;
};

// src/pro.js
var pro = (ref) => new Proxy(ref, {
  get(target, key) {
    return pro(target[key] = target[key] ?? {});
  }
});

// src/refsInfo.js
var refsInfo = (...refs) => {
  const fns = /* @__PURE__ */ new Set();
  refs.forEach((ref) => {
    while (is(Function, ref) && !fns.has(ref) && ref.toString?.()?.includes?.("[native code]")) {
      fns.add(ref);
      ref = Object.getPrototypeOf(ref);
    }
  });
  return Array.from(fns.values()).map(
    /** @return {[string, string, string[]]} */
    (fn) => [
      fn.name,
      Object.getPrototypeOf(fn)?.name ?? "",
      Object.getOwnPropertyNames(fn.prototype ?? /* @__PURE__ */ Object.create(null)).sort()
    ]
  ).sort((a, b) => -(a[0] < b[0]));
};

// src/uuid1.js
var ZEROS = "0".repeat(16);
var counter = 0;
var uuid1 = ({
  date = /* @__PURE__ */ new Date(),
  node = Math.random().toString(16).slice(2)
} = {}) => {
  const time = ZEROS + (1e4 * (+date + 122192928e5)).toString(16);
  counter = counter + 1 & 16383;
  return time.slice(-8).concat(
    "-",
    time.slice(-12, -8),
    // @ts-expect-error
    -1,
    time.slice(-15, -12),
    "-",
    (8 | counter >> 12).toString(16),
    (ZEROS + (counter & 4095).toString(16)).slice(-3),
    "-",
    (ZEROS + node).slice(-12)
  );
};
export {
  chartable,
  eq,
  escape,
  escapeValues,
  fixTypography,
  h,
  has,
  is,
  jcss,
  jsOnParse,
  locale,
  nanolight,
  nanolightJs,
  plUral,
  pro,
  refsInfo,
  s,
  uuid1
};
